@@ -68,7 +68,7 @@ Examples:
 
 Cancel orders by order ID:
 
-    quantrocket blotter cancel -o 6002:45 6001:46
+    quantrocket blotter cancel -d 6002:45 6001:46
 
 Cancel orders by conid:
 
@@ -88,7 +88,7 @@ Cancel all open orders:
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
-        "-o", "--order-ids",
+        "-d", "--order-ids",
         metavar="ORDER_ID",
         nargs="*",
         help="cancel these order IDs")
@@ -117,74 +117,108 @@ Cancel all open orders:
     parser.set_defaults(func="quantrocket.blotter._cli_cancel_orders")
 
     examples = """
-List order status for one or more orders by order ID, conid, order ref, or account.
+Download order statuses.
 
 Examples:
 
-List order status by order ID:
+Download order status by order ID and save to file:
 
-    quantrocket blotter status -o 6002:45 6001:46
+    quantrocket blotter status -d 6002:45 6001:46 -o statuses.csv
 
-List order status for all open orders:
+Download order status for all open orders and display in terminal:
 
-    quantrocket blotter status --open
+    quantrocket blotter status --open | csvlook
 
-List order status of open orders by conid:
+Download order status with extra fields and display as YAML:
+
+    quantrocket blotter status --open --fields Exchange LmtPrice --json | json2yaml
+
+Download order status of open orders by conid:
 
     quantrocket blotter status -i 123456 --open
 
-List order status of open orders by order ref:
+Download order status of open orders by order ref:
 
     quantrocket blotter status --order-refs my-strategy --open
     """
     parser = _subparsers.add_parser(
         "status",
-        help="List order status for one or more orders by order ID, conid, "
-        "order ref, or account",
+        help="download order statuses",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
-    parser.add_argument(
-        "-o", "--order-ids",
+    filters = parser.add_argument_group("filtering options")
+    filters.add_argument(
+        "-d", "--order-ids",
         metavar="ORDER_ID",
         nargs="*",
         help="limit to these order IDs")
-    parser.add_argument(
+    filters.add_argument(
         "-i", "--conids",
         type=int,
         nargs="*",
         metavar="CONID",
         help="limit to orders for these conids")
-    parser.add_argument(
+    filters.add_argument(
         "-r", "--order-refs",
         nargs="*",
         metavar="ORDER_REF",
         help="limit to orders for these order refs")
-    parser.add_argument(
+    filters.add_argument(
         "-a", "--accounts",
         nargs="*",
         metavar="ACCOUNT",
         help="limit to orders for these accounts")
-    parser.add_argument(
+    filters.add_argument(
         "--open",
         action="store_true",
         dest="open_orders",
-        help="limit to open orders (default False, must be True if order_ids not provided)")
-    parser.add_argument(
+        help="limit to open orders")
+    filters.add_argument(
+        "-s", "--start-date",
+        metavar="YYYY-MM-DD",
+        help="limit to orders submitted on or after this date")
+    filters.add_argument(
+        "-e", "--end-date",
+        metavar="YYYY-MM-DD",
+        help="limit to orders submitted on or before this date")
+    outputs = parser.add_argument_group("output options")
+    outputs.add_argument(
         "-f", "--fields",
         metavar="FIELD",
         nargs="*",
         help="return these fields in addition to the default fields (pass '?' or any invalid "
         "fieldname to see available fields)")
-    parser.set_defaults(func="quantrocket.blotter._cli_list_order_statuses")
+    outputs.add_argument(
+        "-o", "--outfile",
+        metavar="OUTFILE",
+        dest="filepath_or_buffer",
+        help="filename to write the data to (default is stdout)")
+    outputs.add_argument(
+        "-j", "--json",
+        action="store_const",
+        const="json",
+        dest="output",
+        help="format output as JSON (default is CSV)")
+    parser.set_defaults(func="quantrocket.blotter._cli_download_order_statuses")
 
     examples = """
 Query current positions.
 
+There are two ways to view positions: blotter view (default) and broker view.
+
+The default "blotter view" returns positions by account, conid, and order ref. Positions
+are tracked based on execution records saved to the blotter database.
+
+"Broker view" (using the `--broker` option) returns positions by account and conid (but
+not order ref) as reported directly by IB. Broker view is more authoritative but less
+informative than blotter view. Broker view is typically used to verify the accuracy
+of blotter view.
+
 Examples:
 
-Query current positions in human-readable format:
+Query current positions:
 
-    quantrocket blotter positions --pretty
+    quantrocket blotter positions
 
 Save current positions to CSV file:
 
@@ -193,6 +227,10 @@ Save current positions to CSV file:
 Query positions for a single order ref:
 
     quantrocket blotter positions --order-refs my-strategy
+
+Query positions using broker view:
+
+    quantrocket blotter positions --broker
     """
     parser = _subparsers.add_parser(
         "positions",
@@ -200,6 +238,70 @@ Query positions for a single order ref:
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     filters = parser.add_argument_group("filtering options")
+    filters.add_argument(
+        "-i", "--conids",
+        type=int,
+        nargs="*",
+        metavar="CONID",
+        help="limit to these conids")
+    filters.add_argument(
+        "-r", "--order-refs",
+        nargs="*",
+        metavar="ORDER_REF",
+        help="limit to these order refs (not supported with broker view)")
+    filters.add_argument(
+        "-a", "--accounts",
+        nargs="*",
+        metavar="ACCOUNT",
+        help="limit to these accounts")
+    filters.add_argument(
+        "--diff",
+        action="store_true",
+        help="limit to positions where the blotter quantity and broker quantity "
+        "disagree (requires --broker)")
+    outputs = parser.add_argument_group("output options")
+    outputs.add_argument(
+        "--broker",
+        action="store_const",
+        dest="view",
+        const="broker",
+        help="return 'broker' view of positions (by account and conid) instead "
+        "of default 'blotter' view (by account, conid, and order ref)")
+    outputs.add_argument(
+        "-o", "--outfile",
+        metavar="OUTFILE",
+        dest="filepath_or_buffer",
+        help="filename to write the data to (default is stdout)")
+    outputs.add_argument(
+        "-j", "--json",
+        action="store_const",
+        const="json",
+        dest="output",
+        help="format output as JSON (default is CSV)")
+    parser.set_defaults(func="quantrocket.blotter._cli_download_positions")
+
+    examples = """
+Generate orders to close positions.
+
+Doesn't actually place any orders but returns an orders file that can be placed
+separately. Additional order parameters can be appended with the `--params` option.
+
+Examples:
+
+Generate MKT orders to close positions for a particular strategy:
+
+    quantrocket blotter close --order-refs my-strategy --params OrderType:MKT Tif:DAY Exchange:SMART
+
+Generate orders and also place them:
+
+    quantrocket blotter close -r my-strategy -p OrderType:MKT Tif:DAY Exchange:SMART | quantrocket blotter order -f -
+    """
+    parser = _subparsers.add_parser(
+        "close",
+        help="generate orders to close positions",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    filters = parser.add_argument_group("filtering options")
     filters.add_argument(
         "-i", "--conids",
         type=int,
@@ -222,20 +324,20 @@ Query positions for a single order ref:
         metavar="OUTFILE",
         dest="filepath_or_buffer",
         help="filename to write the data to (default is stdout)")
-    output_format_group = outputs.add_mutually_exclusive_group()
-    output_format_group.add_argument(
+    outputs.add_argument(
+        "-p", "--params",
+        nargs="*",
+        type=dict_str,
+        metavar="PARAM:VALUE",
+        help="additional parameters to append to each row in output "
+        "(pass as 'param:value', for example OrderType:MKT)")
+    outputs.add_argument(
         "-j", "--json",
         action="store_const",
         const="json",
         dest="output",
         help="format output as JSON (default is CSV)")
-    output_format_group.add_argument(
-        "-p", "--pretty",
-        action="store_const",
-        const="txt",
-        dest="output",
-        help="format output in human-readable format (default is CSV)")
-    parser.set_defaults(func="quantrocket.blotter._cli_download_positions")
+    parser.set_defaults(func="quantrocket.blotter._cli_close_positions")
 
     examples = """
 Query executions from the executions database.
@@ -294,19 +396,19 @@ Examples:
 
 Get a Moonchart PDF of all trading performance PNL:
 
-    quantrocket blotter pnl -o pnl.pdf
+    quantrocket blotter pnl -o pnl.pdf --pdf
 
 Get a PDF for a single account and order ref, broken down by conid:
 
-    quantrocket blotter pnl --accounts U12345 --order-refs mystrategy1 --details -o pnl_details.pdf
+    quantrocket blotter pnl --accounts U12345 --order-refs mystrategy1 --details --pdf -o pnl_details.pdf
 
 Get a CSV of performance results for a particular date range:
 
-    quantrocket blotter pnl -s 2018-03-01 -e 2018-06-30 --csv -o pnl_2018Q2.csv
+    quantrocket blotter pnl -s 2018-03-01 -e 2018-06-30 -o pnl_2018Q2.csv
 
 Calculate daily performance as of 4PM Eastern time (instead of the default 11:59:59 UTC):
 
-    quantrocket blotter pnl --time '16:00:00 America/New_York' -o pnl.pdf
+    quantrocket blotter pnl --time '16:00:00 America/New_York' --pdf -o pnl.pdf
     """
     parser = _subparsers.add_parser(
         "pnl",
@@ -333,16 +435,16 @@ Calculate daily performance as of 4PM Eastern time (instead of the default 11:59
     filters.add_argument(
         "-s", "--start-date",
         metavar="YYYY-MM-DD",
-        help="limit to history on or after this date")
+        help="limit to pnl on or after this date")
     filters.add_argument(
         "-e", "--end-date",
         metavar="YYYY-MM-DD",
-        help="limit to history on or before this date")
+        help="limit to pnl on or before this date")
     filters.add_argument(
         "-t", "--time",
         metavar="HH:MM:SS [TZ]",
-        help="time of day with optional timezone to calculate daily PNL (default is "
-        "11:59:59 UTC)")
+        help="time of day (with optional timezone) for which to calculate daily "
+        "PNL (default is 11:59:59 UTC)")
     outputs = parser.add_argument_group("output options")
     outputs.add_argument(
         "-d", "--details",
@@ -351,10 +453,11 @@ Calculate daily performance as of 4PM Eastern time (instead of the default 11:59
         "account/order ref level (only supported for a single account and order ref "
         "at a time)")
     outputs.add_argument(
-        "--csv",
-        action="store_true",
-        help="return a CSV of PNL (default is to return a PDF "
-        "performance tear sheet)")
+        "--pdf",
+        action="store_const",
+        const="pdf",
+        dest="output",
+        help="return a PDF tear sheet of PNL (default is to return a CSV)")
     outputs.add_argument(
         "-o", "--outfile",
         metavar="OUTFILE",
@@ -367,11 +470,3 @@ Calculate daily performance as of 4PM Eastern time (instead of the default 11:59
     #parser.add_argument("-a", "--accounts", nargs="*", metavar="ACCOUNT", help="limit to these accounts")
     #parser.add_argument("-r", "--rules", nargs="*", metavar="KEY:VALUE", help="rollover rules as multiple key-value pairs in relativedelta format (e.g. days=-8) (omit to use rollover rules defined in master service)")
     #parser.set_defaults(func="quantrocket.blotter.rollover_positions")
-
-    #parser = _subparsers.add_parser("close", help="generate orders to close positions")
-    #parser.add_argument("-s", "--strategies", nargs="*", metavar="CODE", help="limit to these strategies")
-    #parser.add_argument("-c", "--conids", nargs="*", metavar="CONID", help="limit to these conids")
-    #parser.add_argument("-a", "--accounts", nargs="*", metavar="ACCOUNT", help="limit to these accounts")
-    #parser.add_argument("-o", "--order", nargs="+", metavar="FIELD:VALUE", help="order details as JSON or as multiple key-value pairs (e.g. orderType:MKT tif:DAY)")
-    #parser.add_argument("--oca", dest="oca_suffix", metavar="SUFFIX", help="create OCA group containing client ID, order ID, and this suffix (run this command multiple times with this option to create OCA orders)")
-    #parser.set_defaults(func="quantrocket.blotter.close_positions")
