@@ -24,13 +24,18 @@ List databases.
 
 Examples:
 
-List all databases:
+List all databases as a flat list:
 
     quantrocket db list
 
-List history databases:
+List all history databases and include details such as file size:
 
-    quantrocket db list history
+    quantrocket db list history --detail
+
+List details for a sharded history database called usa-stk-15min
+and list each shard individually:
+
+    quantrocket db list history usa-stk-15min --detail --expand
     """
     parser = _subparsers.add_parser(
         "list",
@@ -42,6 +47,23 @@ List history databases:
         nargs="?",
         metavar="SERVICE",
         help="only list databases for this service")
+    parser.add_argument(
+        "codes",
+        nargs="*",
+        metavar="DATABASE_CODE",
+        help="only list databases identified by these codes (omit "
+        "to list all databases "
+        "for service)")
+    parser.add_argument(
+        "-d", "--detail",
+        action="store_true",
+        help="return database statistics (default is to return a "
+        "flat list of database names)")
+    parser.add_argument(
+        "-e", "--expand",
+        action="store_true",
+        help="expand sharded databases to include individual shards "
+        "(default is to list sharded databases as a single database)")
     parser.set_defaults(func="quantrocket.db._cli_list_databases")
 
     examples = """
@@ -68,9 +90,49 @@ Download a database called quantrocket.history.nyse.sqlite:
         help="filename to write the database to")
     parser.set_defaults(func="quantrocket.db._cli_download_database")
 
+    examples = """
+Set or show Amazon S3 configuration for pushing and pulling databases to and
+from S3.
+
+See http://qrok.it/h/dbs3 to learn more.
+
+Examples:
+
+Configure S3 (will prompt for secret access key):
+
+    quantrocket db s3config --access-key-id XXXXXXXX --bucket my-bucket
+
+Preserve existing credentials but point to a new bucket:
+
+    quantrocket db s3config --bucket my-other-bucket
+
+Show current configuration:
+
+    quantrocket db s3config
+    """
+    parser = _subparsers.add_parser(
+        "s3config",
+        help="set or show Amazon S3 configuration for pushing and "
+        "pulling databases to and from S3",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-a", "--access-key-id",
+        help="AWS access key ID")
+    parser.add_argument(
+        "-s", "--secret-access-key",
+        help="AWS secret access key (if omitted and access-key-id is provided, "
+        "will be prompted for secret-access-key)")
+    parser.add_argument(
+        "-b", "--bucket",
+        help="the S3 bucket name to push to/pull from")
+    parser.set_defaults(func="quantrocket.db._cli_get_or_set_s3_config")
+
     examples = """
 Push database(s) to Amazon S3.
 
+See http://qrok.it/h/dbs3 to learn more.
+
 Examples:
 
 Push all databases:
@@ -105,6 +167,8 @@ Push a database called quantrocket.history.nyse.sqlite:
     examples = """
 Pull database(s) from Amazon S3 to the db service.
 
+See http://qrok.it/h/dbs3 to learn more.
+
 Examples:
 
 Pull a database stored on S3 as quantrocket.history.nyse.sqlite.gz:
@@ -113,29 +177,35 @@ Pull a database stored on S3 as quantrocket.history.nyse.sqlite.gz:
     """
     parser = _subparsers.add_parser(
         "s3pull",
-        help="pull database(s) from Amazon S3 to the db service",
+        help="pull database(s) from Amazon S3",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
         "service",
         metavar="SERVICE",
-        help="only pull databases for this service (specify 'all' to pull all services)")
+        help="only pull databases for this service (specify 'all' "
+        "to pull all services)")
     parser.add_argument(
         "codes",
         nargs="*",
         metavar="DATABASE_CODE",
-        help="only pull databases identified by these codes (omit to pull all databases "
+        help="only pull databases identified by these codes (omit to "
+        "pull all databases "
         "for service)")
     parser.add_argument(
         "-f", "--force",
         action="store_true",
         default=False,
-        help="overwrite existing database if one exists (default is to fail if one exists)")
+        help="overwrite existing database if one exists (default is to "
+        "fail if one exists)")
     parser.set_defaults(func="quantrocket.db._cli_s3_pull_databases")
 
     examples = """
 Optimize database file(s) to improve performance.
 
+This runs SQLite's 'VACUUM' command, which defragments the .sqlite file
+and reclaims disk space.
+
 Examples:
 
 Optimize all databases:
@@ -150,11 +220,13 @@ Optimize all databases:
     parser.add_argument(
         "service",
         metavar="SERVICE",
-        help="only optimize databases for this service (specify 'all' to optimize all services)")
+        help="only optimize databases for this service (specify 'all' "
+        "to optimize all services)")
     parser.add_argument(
         "codes",
         nargs="*",
         metavar="DATABASE_CODE",
-        help="only optimize databases identified by these codes (omit to optimize all databases "
+        help="only optimize databases identified by these codes (omit "
+        "to optimize all databases "
         "for service)")
     parser.set_defaults(func="quantrocket.db._cli_optimize_databases")
