@@ -53,42 +53,42 @@ List stock exchanges in North America:
     parser.set_defaults(func="quantrocket.master._cli_list_exchanges")
 
     examples = """
-Fetch securities listings from IB into securities master database, either by
-exchange or by universes/conids.
+Collect securities listings from IB and store in securities master database
+(quantrocket.master.main.sqlite).
 
 Specify an exchange (optionally filtering by security type, currency, and/or
-symbol) to fetch listings from the IB website and fetch associated contract
-details from the IB API. Or, specify universes or conids to fetch details from
+symbol) to collect listings from the IB website and collect associated contract
+details from the IB API. Or, specify universes or conids to collect details from
 the IB API, bypassing the website.
 
 Examples:
 
-Fetch all Toronto Stock Exchange stock listings:
+Collect all Toronto Stock Exchange stock listings:
 
-    quantrocket master listings --exchange TSE --sec-types STK
+    quantrocket master collect --exchanges TSE --sec-types STK
 
-Fetch all NYSE ARCA ETF listings:
+Collect all NYSE ARCA ETF listings:
 
-    quantrocket master listings --exchange ARCA --sec-types ETF
+    quantrocket master collect -e ARCA --sec-types ETF
 
-Fetch specific symbols from Nasdaq:
+Collect specific symbols from Nasdaq:
 
-    quantrocket master listings --exchange NASDAQ --symbols AAPL GOOG NFLX
+    quantrocket master collect -e NASDAQ --symbols AAPL GOOG NFLX
 
-Re-fetch contract details for an existing universe called "japan-fin":
+Re-collect contract details for an existing universe called "japan-fin":
 
-    quantrocket master listings --universes "japan-fin"
+    quantrocket master collect --universes "japan-fin"
     """
     parser = _subparsers.add_parser(
-        "listings",
-        help="fetch securities listings from IB into securities master database, either by "
-        "exchange or by universes/conids",
+        "collect",
+        help="collect securities listings from IB and store in securities master database",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
-        "-e", "--exchange",
+        "-e", "--exchanges",
+        nargs="*",
         metavar="EXCHANGE",
-        help="the exchange code to fetch listings for (required unless providing universes "
+        help="one or more exchange codes to collect listings for (required unless providing universes "
         "or conids)")
     parser.add_argument(
         "-t", "--sec-types",
@@ -117,10 +117,40 @@ Re-fetch contract details for an existing universe called "japan-fin":
         nargs="*",
         metavar="CONID",
         help="limit to these conids")
-    parser.set_defaults(func="quantrocket.master._cli_fetch_listings")
+    parser.add_argument(
+        "--exchange",
+        metavar="EXCHANGE",
+        help="DEPRECATED, this option will be removed in a future release, please use "
+        "`exchanges` instead (previously only a single exchange was supported but "
+        "now multiple exchanges are supported)")
+    parser.set_defaults(func="quantrocket.master._cli_collect_listings")
 
     examples = """
-Fetch option chains for underlying securities.
+Collect securities listings from Sharadar and save to
+quantrocket.master.sharadar.sqlite.
+
+Requires a Sharadar data plan. Collects NYSE, NASDAQ, or all US stock
+listings, depending on your plan.
+
+Sharadar listings have their own ConIds which are distinct from IB ConIds.
+To facilitate using Sharadar and IB data together or separately, this command
+also collects a list of IB<->Sharadar ConId translations and saves them
+to quantrocket.master.translations.sqlite. They can be queried via
+`quantrocket master translate`.
+
+Examples:
+
+    quantrocket master collect-sharadar
+    """
+    parser = _subparsers.add_parser(
+        "collect-sharadar",
+        help="collect securities listings from Sharadar and save to quantrocket.master.sharadar.sqlite",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.set_defaults(func="quantrocket.master._cli_collect_sharadar_listings")
+
+    examples = """
+Collect option chains for underlying securities.
 
 Note: option chains often consist of hundreds, sometimes thousands of options
 per underlying security. Be aware that requesting option chains for large
@@ -131,41 +161,41 @@ hundred megabytes, and potentially add latency to database queries.
 
 Examples:
 
-Fetch option chains for several underlying securities:
+Collect option chains for several underlying securities:
 
     quantrocket master options --conids 8314 208813720 107113386
 
-Fetch option chains for NQ futures:
+Collect option chains for NQ futures:
 
     quantrocket master get -e GLOBEX -s NQ -t FUT | quantrocket master options -f -
 
-Fetch option chains for a large universe of stocks called "nyse-stk" (see note above):
+Collect option chains for a large universe of stocks called "nyse-stk" (see note above):
 
     quantrocket master options -u "nyse-stk"
     """
     parser = _subparsers.add_parser(
         "options",
-        help="fetch option chains for underlying securities",
+        help="collect option chains for underlying securities",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
         "-u", "--universes",
         nargs="*",
         metavar="UNIVERSE",
-        help="fetch options for these universes of underlying securities")
+        help="collect options for these universes of underlying securities")
     parser.add_argument(
         "-i", "--conids",
         type=int,
         nargs="*",
         metavar="CONID",
-        help="fetch options for these underlying conids")
+        help="collect options for these underlying conids")
     parser.add_argument(
         "-f", "--infile",
         metavar="INFILE",
         dest="infilepath_or_buffer",
-        help="fetch options for the conids in this file (specify '-' to read "
+        help="collect options for the conids in this file (specify '-' to read "
         "file from stdin)")
-    parser.set_defaults(func="quantrocket.master._cli_fetch_option_chains")
+    parser.set_defaults(func="quantrocket.master._cli_collect_option_chains")
 
     examples = """
 Query security details from the securities master database and download to
@@ -183,9 +213,14 @@ Download a CSV of all ARCA ETFs and use it to create a universe called
 
     quantrocket master get --exchanges ARCA --sec-types ETF | quantrocket master universe "arca-etf" --infile -
 
-Pretty print the exchange and currency for all listings of AAPL:
+Query the exchange and currency for all listings of AAPL and format for
+terminal display:
 
-    quantrocket master get --symbols AAPL --fields PrimaryExchange Currency --pretty
+    quantrocket master get --symbols AAPL --fields PrimaryExchange Currency | csvlook -I
+
+Download a CSV of Sharadar securities from quantrocket.master.sharadar.sqlite:
+
+    quantrocket master get --domain sharadar -o sharadar_securities.csv
     """
     parser = _subparsers.add_parser(
         "get",
@@ -252,10 +287,17 @@ Pretty print the exchange and currency for all listings of AAPL:
         metavar="CATEGORY",
         help="limit to these categories")
     filters.add_argument(
-        "-d", "--delisted",
+        "--exclude-delisted",
         action="store_true",
         default=False,
-        help="include delisted securities")
+        help="exclude delisted securities (default is to include them)")
+    filters.add_argument(
+        "--delisted",
+        action="store_true",
+        default=True,
+        help="[DEPRECATED] include delisted securities; this parameter is "
+        "deprecated and will be removed in a future release; it has no effect "
+        "as delisted securities are included by default")
     filters.add_argument(
         "-m", "--frontmonth",
         action="store_true",
@@ -286,11 +328,17 @@ Pretty print the exchange and currency for all listings of AAPL:
         nargs="*",
         help="only return these fields (pass '?' or any invalid fieldname to see "
         "available fields)")
+    domains = parser.add_argument_group("domain options")
+    domains.add_argument(
+        "-d", "--domain",
+        choices=["main", "sharadar"],
+        help="query against this domain (default is 'main', which runs against "
+        "quantrocket.master.main.sqlite. Possible choices: %(choices)s)")
     parser.set_defaults(func="quantrocket.master._cli_download_master_file")
 
     examples = """
 Flag security details that have changed in IB's system since the time they
-were last loaded into the securities master database.
+were last collected into the securities master database.
 
 Diff can be run synchronously or asynchronously (asynchronous is the default
 and is recommended if diffing more than a handful of securities).
@@ -324,7 +372,7 @@ or that are now associated with the PINK exchange:
     parser = _subparsers.add_parser(
         "diff",
         help="flag security details that have changed in IB's system since the time "
-        "they were last loaded into the securities master database",
+        "they were last collected into the securities master database",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
@@ -366,6 +414,45 @@ or that are now associated with the PINK exchange:
         "asynchronously and log the results, if any, to flightlog")
     parser.set_defaults(func="quantrocket.master._cli_diff_securities")
 
+    examples = """
+Translate conids (contract IDs) from one domain to another.
+
+Only translations to and from the "main" domain (that is, the
+IB domain) are supported.
+
+Examples:
+
+Translate IB conids to Sharadar conids:
+
+    quantrocket master translate 12345 23456 --to sharadar
+
+Translate Sharadar conids to IB conids:
+
+    quantrocket master translate 98765 87654 --from sharadar
+    """
+    parser = _subparsers.add_parser(
+        "translate",
+        help="translate conids from one domain to another",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "conids",
+        nargs="+",
+        metavar="CONID",
+        help="the conids to translate")
+    parser.add_argument(
+        "--from",
+        choices=["main","sharadar"],
+        dest="from_domain",
+        help="the domain to translate from. This is the domain of the provided "
+        "conids. Possible choices: %(choices)s")
+    parser.add_argument(
+        "--to",
+        choices=["main","sharadar"],
+        dest="to_domain",
+        help="the domain to translate to. Possible choices: %(choices)s")
+    parser.set_defaults(func="quantrocket.master._cli_translate_conids")
+
     examples = """
 List universes and their size.
 
@@ -374,12 +461,22 @@ Examples:
 List all universes and their size:
 
     quantrocket master list-universes
+
+List universes in quantrocket.master.sharadar.sqlite:
+
+    quantrocket master list-universes --domain sharadar
     """
     parser = _subparsers.add_parser(
         "list-universes",
         help="list universes and their size",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-d", "--domain",
+        choices=["main","sharadar"],
+        help="the domain to list universes for (default is 'main', which "
+        "runs against quantrocket.master.main.sqlite. Possible choices: "
+        "%(choices)s)")
     parser.set_defaults(func="quantrocket.master._cli_list_universes")
 
     examples = """
@@ -405,6 +502,10 @@ Create a universe consisting of several existing universes:
 Copy a universe but exclude delisted securities:
 
     quantrocket master universe "hong-kong-active" --from-universes "hong-kong" --exclude-delisted
+
+Create a universe of all listings in quantrocket.master.sharadar.sqlite:
+
+    quantrocket master get --domain sharadar | quantrocket master universe "sharadar-stk" --domain sharadar -f -
     """
     parser = _subparsers.add_parser(
         "universe",
@@ -440,28 +541,44 @@ Copy a universe but exclude delisted securities:
         "-r", "--replace",
         action="store_true",
         help="replace universe if universe already exists")
+    parser.add_argument(
+        "-d", "--domain",
+        choices=["main", "sharadar"],
+        help="create universe in this domain (default is 'main', which runs against "
+        "quantrocket.master.main.sqlite. Possible choices: %(choices)s)")
     parser.set_defaults(func="quantrocket.master._cli_create_universe")
 
     examples = """
-Delete a universe. (The listings details of the member securities won't be
-deleted, only their grouping as a universe).
+Delete a universe.
+
+The listings details of the member securities won't be deleted, only their
+grouping as a universe.
 
 Examples:
 
 Delete the universe called "italy-stk" (the listings details of the member
 securities won't be deleted, only their grouping as a universe):
 
-    quantrocket master delete-universe "italy-stk"
+    quantrocket master delete-universe 'italy-stk'
+
+Delete a universe from the sharadar domain (quantrocket.master.sharadar.sqlite):
+
+    quantrocket master delete-universe 'sharadar-nyse-delisted' --domain sharadar
     """
     parser = _subparsers.add_parser(
         "delete-universe",
-        help="delete a universe (the listings details of the member securities won't "
-        "be deleted, only their grouping as a universe)",
+        help="delete a universe",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
         "code",
         help="the universe code")
+    parser.add_argument(
+        "-d", "--domain",
+        choices=["main","sharadar"],
+        help="the domain from which to delete the universe (default is "
+        "'main', which runs against quantrocket.master.main.sqlite. "
+        "Possible choices: %(choices)s)")
     parser.set_defaults(func="quantrocket.master._cli_delete_universe")
 
     examples = """
@@ -532,17 +649,17 @@ Delist a security by symbol + exchange:
     parser.set_defaults(func="quantrocket.master._cli_delist_security")
 
     examples = """
-Fetch upcoming trading hours for exchanges and save to securites master database.
+Collect upcoming trading hours for exchanges and save to securites master database.
 
 Examples:
 
-Fetch trading hours for all exchanges in securities master database:
+Collect trading hours for all exchanges in securities master database:
 
-    quantrocket master fetch-calendar
+    quantrocket master collect-calendar
     """
     parser = _subparsers.add_parser(
-        "fetch-calendar",
-        help="fetch upcoming trading hours for exchanges and save to securites master database",
+        "collect-calendar",
+        help="collect upcoming trading hours for exchanges and save to securites master database",
         epilog=examples,
         formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument(
@@ -550,7 +667,7 @@ Fetch trading hours for all exchanges in securities master database:
         nargs="*",
         metavar="EXCHANGE",
         help="limit to these exchanges")
-    parser.set_defaults(func="quantrocket.master._cli_fetch_calendar")
+    parser.set_defaults(func="quantrocket.master._cli_collect_calendar")
 
     examples = """
 Check whether exchanges are open or closed.
@@ -615,9 +732,9 @@ Place Moonshot orders if NYSE is open now:
 
     quantrocket master isopen NYSE && quantrocket moonshot orders my-strategy | quantrocket blotter order -f -
 
-Fetch historical data for Australian stocks if the exchange was open 4 hours ago:
+Collect historical data for Australian stocks if the exchange was open 4 hours ago:
 
-    quantrocket master isopen ASX --ago 4h && quantrocket history fetch asx-stk-1d
+    quantrocket master isopen ASX --ago 4h && quantrocket history collect asx-stk-1d
 
 Log a message if the London Stock Exchange will be open in 30 minutes:
 
@@ -687,10 +804,10 @@ Place Moonshot orders if the NYSE will be closed NYSE in 1 hour:
 
     quantrocket master isclosed NYSE --in 1h && quantrocket moonshot orders my-strategy | quantrocket blotter order -f -
 
-Fetch historical data for Australian stocks if the exchange is closed now but was
+Collect historical data for Australian stocks if the exchange is closed now but was
 open 4 hours ago:
 
-    quantrocket master isclosed ASX && quantrocket master isopen ASX --ago 4h && quantrocket history fetch asx-stk-1d
+    quantrocket master isclosed ASX && quantrocket master isopen ASX --ago 4h && quantrocket history collect asx-stk-1d
 
 Place Moonshot orders if the NYSE has been closed since month end:
 
@@ -803,3 +920,66 @@ Round the LmtPrice column in a CSV of Moonshot orders then place the orders:
         dest="outfilepath_or_buffer",
         help="filename to write the data to (default is stdout)")
     parser.set_defaults(func="quantrocket.master._cli_round_to_tick_sizes")
+
+    examples = """
+Collect upcoming trading hours for exchanges and save to securites master database.
+
+[DEPRECATED] `fetch-calendar` is deprecated and will be removed in a future release,
+please use `collect-calendar` instead.
+    """
+    parser = _subparsers.add_parser(
+        "fetch-calendar",
+        help="[DEPRECATED] collect upcoming trading hours for exchanges and save to securites master database",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-e", "--exchanges",
+        nargs="*",
+        metavar="EXCHANGE",
+        help="limit to these exchanges")
+    parser.set_defaults(func="quantrocket.master._cli_fetch_calendar")
+
+    examples = """
+Collect securities listings from IB and store in securities master database.
+
+[DEPRECATED] `listings` is deprecated and will be removed in a future release,
+please use `collect` instead.
+    """
+    parser = _subparsers.add_parser(
+        "listings",
+        help="[DEPRECATED] collect securities listings from IB and store in securities master database",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-e", "--exchange",
+        metavar="EXCHANGE",
+        help="the exchange code to collect listings for (required unless providing universes "
+        "or conids)")
+    parser.add_argument(
+        "-t", "--sec-types",
+        nargs="*",
+        metavar="SEC_TYPE",
+        choices=["STK", "ETF", "FUT", "CASH", "IND"],
+        help="limit to these security types. Possible choices: %(choices)s")
+    parser.add_argument(
+        "-c", "--currencies",
+        nargs="*",
+        metavar="CURRENCY",
+        help="limit to these currencies")
+    parser.add_argument(
+        "-s", "--symbols",
+        nargs="*",
+        metavar="SYMBOL",
+        help="limit to these symbols")
+    parser.add_argument(
+        "-u", "--universes",
+        nargs="*",
+        metavar="UNIVERSE",
+        help="limit to these universes")
+    parser.add_argument(
+        "-i", "--conids",
+        type=int,
+        nargs="*",
+        metavar="CONID",
+        help="limit to these conids")
+    parser.set_defaults(func="quantrocket.master._cli_listings")
\ No newline at end of file
