@@ -41,6 +41,10 @@ Backtest a single strategy called demo using all available history and return a
 PDF tear sheet:
 
     quantrocket moonshot backtest demo --pdf -o tearsheet.pdf
+
+Run a backtest in 1-year segments to reduce memory usage:
+
+    quantrocket moonshot backtest big-strategy -s 2000-01-01 -e 2018-01-01 --segment A -o results.csv
     """
     parser = _subparsers.add_parser(
         "backtest",
@@ -61,6 +65,12 @@ PDF tear sheet:
         "-e", "--end-date",
         metavar="YYYY-MM-DD",
         help="the backtest end date (default is to use all available history)")
+    backtest_options.add_argument(
+        "-g", "--segment",
+        metavar="FREQ",
+        help="backtest in date segments of this size, to reduce memory usage "
+        "(use Pandas frequency string, e.g. 'A' for annual segments or 'Q' "
+        "for quarterly segments)")
     backtest_options.add_argument(
         "-l", "--allocations",
         type=dict_str,
@@ -118,6 +128,10 @@ called trend-friend and return a PDF:
 Run a 2-D parameter scan for multiple strategies and return a PDF:
 
     quantrocket moonshot paramscan strat1 strat2 strat3 -p MIN_STD -v 1 1.5 2 --param2 STD_WINDOW --vals2 20 50 100 200 --pdf -o tearsheet.pdf
+
+Run a parameter scan in 1-year segments to reduce memory usage:
+
+    quantrocket moonshot paramscan big-strategy -s 2000-01-01 -e 2018-01-01 --segment A -p MAVG_WINDOW -v 20 50 100 --pdf -o tearsheet.pdf
     """
     parser = _subparsers.add_parser(
         "paramscan",
@@ -138,6 +152,12 @@ Run a 2-D parameter scan for multiple strategies and return a PDF:
         "-e", "--end-date",
         metavar="YYYY-MM-DD",
         help="the backtest end date (default is to use all available history)")
+    backtest_options.add_argument(
+        "-g", "--segment",
+        metavar="FREQ",
+        help="backtest in date segments of this size, to reduce memory usage "
+        "(use Pandas frequency string, e.g. 'A' for annual segments or 'Q' "
+        "for quarterly segments)")
     backtest_options.add_argument(
         "-p", "--param1",
         metavar="PARAM",
@@ -202,6 +222,138 @@ Run a 2-D parameter scan for multiple strategies and return a PDF:
         help="the location to write the results file (omit to write to stdout)")
     parser.set_defaults(func="quantrocket.moonshot._cli_scan_parameters")
 
+    examples = """
+Run a walk-forward backtest of a machine learning strategy.
+
+The date range will be split into segments of `--train` size. For each
+segment, the model will be trained with the data, then the trained model will
+be backtested on the following segment.
+
+By default, uses scikit-learn's StandardScaler+SGDRegressor. Also supports other
+scikit-learn models/pipelines and Keras models. To customize model, instantiate
+the model locally, serialize it to disk, and pass the filename of the serialized
+model as `--model`.
+
+Supports expanding walk-forward backtests (the default), which use an anchored start date
+for model training, or rolling walk-forward backtests (by specifying `--rolling-train`),
+which use a rolling or non-anchored start date for model training.
+
+Returns a backtest results CSV and a dump of the machine learning model
+as of the end of the analysis.
+
+Examples:
+
+Run a walk-forward backtest using the default model and retrain the model annually,
+writing the backtest results and trained model to demo_ml_results.csv and
+demo_ml_trained_model.joblib, respectively:
+
+    quantrocket moonshot ml-walkforward demo-ml -s 2007-01-01 -e 2018-12-31 --train A -o demo_ml*
+
+Run a walk-forward backtest using a custom model (serialized with joblib), retrain the
+model annually, don't perform backtesting until after 5 years of initial training,
+and further split the training and backtesting into quarterly segments to reduce
+memory usage:
+
+    quantrocket moonshot ml-walkforward demo-ml -s 2007-01-01 -e 2018-12-31 --model my_model.joblib --train A --min-train 5Y --segment Q -o demo_ml*
+    """
+    parser = _subparsers.add_parser(
+        "ml-walkforward",
+        help="run a walk-forward backtest of a machine learning strategy",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "strategy",
+        metavar="CODE",
+        help="the strategy code")
+    walkforward_options = parser.add_argument_group("walk-forward analysis options")
+    walkforward_options.add_argument(
+        "-s", "--start-date",
+        metavar="YYYY-MM-DD",
+        required=True,
+        help="the analysis start date (note that model training will start on this date "
+        "but backtest will not start until after the initial training period)")
+    walkforward_options.add_argument(
+        "-e", "--end-date",
+        metavar="YYYY-MM-DD",
+        required=True,
+        help="the analysis end date")
+    walkforward_options.add_argument(
+        "-t", "--train",
+        metavar="FREQ",
+        required=True,
+        help="train model this frequently (use Pandas frequency string, e.g. 'A' "
+        "for annual training or 'Q' for quarterly training)")
+    walkforward_options.add_argument(
+        "-m", "--min-train",
+        metavar="FREQ",
+        help="don't backtest until at least this much model training has occurred; "
+        "defaults to the length of `--train` if not specified (use Pandas frequency "
+        "string, e.g. '5Y' for 5 years of initial training)")
+    walkforward_options.add_argument(
+        "-r", "--rolling-train",
+        metavar="FREQ",
+        help="train model with a rolling window of this length; if omitted, train "
+        "model with an expanding window (use Pandas frequency string, e.g. '3Y' for "
+        "a 3-year rolling training window)")
+    walkforward_options.add_argument(
+        "-f", "--model",
+        dest="model_filepath",
+        help="filepath of serialized model to use, filename must end in '.joblib' or "
+        "'.pkl' (if omitted, default model is scikit-learn's StandardScaler+SGDRegressor)")
+    walkforward_options.add_argument(
+        "--force-nonincremental",
+        action="store_true",
+        help="force the model to be trained non-incrementally (i.e. load entire training "
+        "data set into memory) even if it supports incremental learning. Required "
+        "in order to perform a rolling (as opposed to expanding) walk-forward backtest "
+        "with a model that supports incremental learning.")
+    backtest_options = parser.add_argument_group("backtest options")
+    backtest_options.add_argument(
+        "-g", "--segment",
+        metavar="FREQ",
+        help="train and backtest in date segments of this size, to reduce memory "
+        "usage; must be smaller than `--train`/`--min-train` or will have no effect "
+        "(use Pandas frequency string, e.g. 'A' for annual segments or 'Q' for "
+        "quarterly segments)")
+    backtest_options.add_argument(
+        "-l", "--allocation",
+        type=float,
+        metavar="FLOAT",
+        help="the allocation for the strategy (default 1.0)")
+    backtest_options.add_argument(
+        "-n", "--nlv",
+        nargs="*",
+        type=dict_str,
+        metavar="CURRENCY:NLV",
+        help="the NLV (net liquidation value, i.e. account balance) to assume for "
+        "the backtest, expressed in each currency represented in the backtest (pass "
+        "as 'currency:nlv')")
+    backtest_options.add_argument(
+        "-p", "--params",
+        nargs="*",
+        type=dict_str,
+        metavar="PARAM:VALUE",
+        help="one or more strategy params to set on the fly before backtesting "
+        "(pass as 'param:value')")
+    outputs = parser.add_argument_group("output options")
+    outputs.add_argument(
+        "-d", "--details",
+        action="store_true",
+        help="return detailed results for all securities instead of aggregating")
+    outputs.add_argument(
+        "--progress",
+        action="store_true",
+        help="log status and Sharpe ratios of each walk-forward segment during "
+        "analysis (default False)")
+    outputs.add_argument(
+        "-o", "--outfile",
+        metavar="FILEPATH",
+        dest="filepath_or_buffer",
+        help="the location to write the ZIP file to; or, if path ends with '*', the "
+        "pattern to use for extracting the zipped files. For example, if the path is "
+        "my_ml*, files will extracted to my_ml_results.csv and my_ml_trained_model.joblib.")
+    parser.set_defaults(func="quantrocket.moonshot._cli_ml_walkforward")
+
     examples = """
 Run one or more strategies and generate orders.
 
@@ -246,7 +398,9 @@ Generate orders as if it were an earlier date (for prupose of review):
          help="generate orders as if it were this date, rather than using today's date")
     parser.add_argument(
         "-j", "--json",
-        action="store_true",
+        action="store_const",
+        const="json",
+        dest="output",
         help="format orders as JSON (default is CSV)")
     parser.add_argument(
         "-o", "--outfile",
