@@ -12,20 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import io
+import six
 import os
 import sys
 import time
+import requests
 from quantrocket.master import download_master_file
 from quantrocket.houston import houston
 from quantrocket.cli.utils.output import json_to_cli
 from quantrocket.cli.utils.stream import to_bytes
 from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
-from quantrocket.exceptions import ParameterError
+from quantrocket.exceptions import ParameterError, NoHistoricalData
+from quantrocket.utils.warn import deprecated_replaced_by
 
 TMP_DIR = os.environ.get("QUANTROCKET_TMP_DIR", "/tmp")
 
-def create_db(code, universes=None, start_date=None, end_date=None,
+def create_db(code, universes=None, conids=None, start_date=None, end_date=None,
               vendor=None, bar_size=None, bar_type=None, outside_rth=False,
               primary_exchange=False, times=None, between_times=None,
               shard=None, no_config=False, config_filepath_or_buffer=None):
@@ -40,19 +42,21 @@ def create_db(code, universes=None, start_date=None, end_date=None,
     universes : list of str
         include these universes
 
+    conids : list of int
+        include these conids
+
     start_date : str (YYYY-MM-DD), optional
-        fetch history back to this start date (default is to fetch as far back as data
+        collect history back to this start date (default is to collect as far back as data
         is available)
 
     end_date : str (YYYY-MM-DD), optional
-        fetch history up to this end date (default is to fetch up to the present)
+        collect history up to this end date (default is to collect up to the present)
 
     vendor : str, optional
-        the vendor to fetch data from (defaults to 'ib' which is currently the only
-        supported vendor)
+        the vendor to collect data from (default 'ib'. Possible choices: ib, sharadar)
 
     bar_size : str, required for vendor ib
-        the bar size to fetch. Possible choices:
+        the bar size to collect. Possible choices:
         "1 secs", "5 secs",	"10 secs", "15 secs", "30 secs",
         "1 min", "2 mins", "3 mins", "5 mins", "10 mins", "15 mins", "20 mins", "30 mins",
         "1 hour", "2 hours", "3 hours", "4 hours", "8 hours",
@@ -61,7 +65,7 @@ def create_db(code, universes=None, start_date=None, end_date=None,
         "1 month"
 
     bar_type : str, optional
-        the bar type to fetch (if not specified, defaults to MIDPOINT for forex and
+        the bar type to collect (if not specified, defaults to MIDPOINT for forex and
         TRADES for everything else). Possible choices:
         "TRADES",
         "ADJUSTED_LAST",
@@ -88,12 +92,15 @@ def create_db(code, universes=None, start_date=None, end_date=None,
         mutually exclusive with `times`)
 
     shard : str, optional
-        whether and how to shard the database, i.e. break it into smaller pieces. Possible
-        choices are `time` (separate database for each bar time), `off` (no sharding), or
-        `auto` (decide automatically based on bar size and universe size). Default `auto`.
+        whether and how to shard the database, i.e. break it into smaller pieces.
+        Required for intraday databases. Possible choices are `year` (separate
+        database for each year), `time` (separate database for each bar time),
+        `conid` (separate database for each security), `conid,time` (duplicate copies
+        of database, one sharded by conid and the other by time), or `off` (no
+        sharding). See http://qrok.it/h/shard for more help.
 
     no_config : bool
-        create a database with no config (data can be loaded manually instead of fetched
+        create a database with no config (data can be loaded manually instead of collected
         from a vendor)
 
     config_filepath_or_buffer : str or file-like object, optional
@@ -108,6 +115,8 @@ def create_db(code, universes=None, start_date=None, end_date=None,
     params = {}
     if universes:
         params["universes"] = universes
+    if conids:
+        params["conids"] = conids
     if start_date:
         params["start_date"] = start_date
     if end_date:
@@ -201,38 +210,38 @@ def drop_db(code, confirm_by_typing_db_code_again=None):
 def _cli_drop_db(*args, **kwargs):
     return json_to_cli(drop_db, *args, **kwargs)
 
-def fetch_history(codes, priority=False, conids=None, universes=None,
-                  start_date=None, end_date=None, availability_only=False,
-                  delist_missing=False):
+def collect_history(codes, priority=False, conids=None, universes=None,
+                    start_date=None, end_date=None, availability_only=False,
+                    delist_missing=False):
     """
-    Fetch historical market data from IB and save it to a history database. The request is
-    queued and the data is fetched asynchronously.
+    Collect historical market data from IB and save it to a history database. The request is
+    queued and the data is collected asynchronously.
 
     Parameters
     ----------
     codes : list of str, required
-        the database code(s) to fetch data for
+        the database code(s) to collect data for
 
     priority : bool
         use the priority queue (default is to use the standard queue)
 
     conids : list of int, optional
-        fetch history for these conids, overriding config (typically
-        used to fetch a subset of securities)
+        collect history for these conids, overriding config (typically
+        used to collect a subset of securities)
 
     universes : list of str, optional
-        fetch history for these universes, overriding config (typically
-        used to fetch a subset of securities)
+        collect history for these universes, overriding config (typically
+        used to collect a subset of securities)
 
     start_date : str (YYYY-MM-DD), optional
-        fetch history back to this start date, overriding config
+        collect history back to this start date, overriding config
 
     end_date : str (YYYY-MM-DD), optional
-        fetch history up to this end date, overriding config
+        collect history up to this end date, overriding config
 
     availability_only : bool
         determine and store how far back data is available but
-        don't yet fetch the data
+        don't yet collect the data
 
     delist_missing : bool
         auto-delist securities that are no longer available from IB
@@ -265,12 +274,12 @@ def fetch_history(codes, priority=False, conids=None, universes=None,
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def _cli_fetch_history(*args, **kwargs):
-    return json_to_cli(fetch_history, *args, **kwargs)
+def _cli_collect_history(*args, **kwargs):
+    return json_to_cli(collect_history, *args, **kwargs)
 
 def get_history_queue():
     """
-    Get the current queue of historical data requests.
+    Get the current queue of historical data collections.
 
     Returns
     -------
@@ -285,17 +294,17 @@ def get_history_queue():
 def _cli_get_history_queue(*args, **kwargs):
     return json_to_cli(get_history_queue, *args, **kwargs)
 
-def cancel_history_requests(codes, queues=None):
+def cancel_collections(codes, queues=None):
     """
-    Cancel running or pending historical data requests.
+    Cancel running or pending historical data collections.
 
     Parameters
     ----------
     codes : list of str, required
-        the database code(s) to cancel requests for
+        the database code(s) to cancel collections for
 
     queues : list of str, optional
-        only cancel requests in these queues. Possible choices: standard, priority
+        only cancel collections in these queues. Possible choices: standard, priority
 
     Returns
     -------
@@ -312,8 +321,38 @@ def cancel_history_requests(codes, queues=None):
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def _cli_cancel_history_requests(*args, **kwargs):
-    return json_to_cli(cancel_history_requests, *args, **kwargs)
+def _cli_cancel_collections(*args, **kwargs):
+    return json_to_cli(cancel_collections, *args, **kwargs)
+
+def wait_for_collections(codes, timeout=None):
+    """
+    Wait for historical data collection to finish.
+
+    Parameters
+    ----------
+    codes : list of str, required
+        the database code(s) to wait for
+
+    timeout : str, optional
+        time out if data collection hasn't finished after this much time (use Pandas
+        timedelta string, e.g. 30sec or 5min or 2h)
+
+    Returns
+    -------
+    dict
+        status message
+
+    """
+    params = {}
+    params["codes"] = codes
+    if timeout:
+        params["timeout"] = timeout
+    response = houston.put("/history/queue", params=params, timeout=60*60*24*365)
+    houston.raise_for_status_with_json(response)
+    return response.json()
+
+def _cli_wait_for_collections(*args, **kwargs):
+    return json_to_cli(wait_for_collections, *args, **kwargs)
 
 def download_history_availability_file(code, filepath_or_buffer=None, output="csv"):
     """
@@ -321,7 +360,7 @@ def download_history_availability_file(code, filepath_or_buffer=None, output="cs
 
     This function is normally called after running:
 
-        quantrocket history fetch mydb --availability
+        quantrocket history collect mydb --availability
 
     Parameters
     ----------
@@ -361,11 +400,11 @@ def get_history_availability(code):
     """
     Query historical data availability from a history database, returning a
     Series of start dates (with conids as the index) representing how far back
-    data can be fetched from IB.
+    data can be collected from IB.
 
     This function is normally called after running a command such as:
 
-        quantrocket history fetch [DB] --availability
+        quantrocket history collect [DB] --availability
 
     Parameters
     ----------
@@ -398,7 +437,7 @@ def get_history_availability(code):
     except ImportError:
         raise ImportError("pandas must be installed to use this function")
 
-    f = io.StringIO()
+    f = six.StringIO()
     download_history_availability_file(code, f)
     start_dates = pd.read_csv(f, index_col="ConId", parse_dates=["StartDate"])
     return start_dates.StartDate
@@ -501,7 +540,13 @@ def download_history_file(code, filepath_or_buffer=None, output="csv",
     response = houston.get("/history/{0}.{1}".format(code, output), params=params,
                            timeout=60*30)
 
-    houston.raise_for_status_with_json(response)
+    try:
+        houston.raise_for_status_with_json(response)
+    except requests.HTTPError as e:
+        # Raise a dedicated exception
+        if "no history matches the query parameters" in repr(e).lower():
+            raise NoHistoricalData(e)
+        raise
 
     filepath_or_buffer = filepath_or_buffer or sys.stdout
 
@@ -510,31 +555,12 @@ def download_history_file(code, filepath_or_buffer=None, output="csv",
 def _cli_download_history_file(*args, **kwargs):
     return json_to_cli(download_history_file, *args, **kwargs)
 
-def _infer_timezone(prices):
-    """
-    Infers the timezone from the component securities if possible.
-    """
-    if "Timezone" not in prices.index.get_level_values("Field"):
-        raise ParameterError(
-            "cannot infer timezone because Timezone field is missing, "
-            "please specify timezone or include Timezone in master_fields")
-
-    timezones = prices.loc["Timezone"].stack().unique()
-
-    if len(timezones) > 1:
-        raise ParameterError(
-            "cannot infer timezone because multiple timezones are present "
-            "in data, please specify timezone explicitly (timezones: {0})".format(
-                ", ".join(timezones)))
-
-    return timezones[0]
-
 def get_historical_prices(codes, start_date=None, end_date=None,
                           universes=None, conids=None,
                           exclude_universes=None, exclude_conids=None,
                           times=None, cont_fut=None, fields=None,
-                          master_fields=None, timezone=None,
-                          infer_timezone=None):
+                          timezone=None, infer_timezone=None,
+                          master_fields=None):
     """
     Query one or more history databases and load prices into a DataFrame.
 
@@ -577,10 +603,6 @@ def get_historical_prices(codes, start_date=None, end_date=None,
         only return these fields (pass ['?'] or any invalid fieldname to see
         available fields)
 
-    master_fields : list of str, optional
-        append these fields from the securities master database (pass ['?'] or any
-        invalid fieldname to see available fields)
-
     timezone : str, optional
         convert timestamps to this timezone, for example America/New_York (see
         `pytz.all_timezones` for choices); ignored for non-intraday bar sizes
@@ -590,6 +612,13 @@ def get_historical_prices(codes, start_date=None, end_date=None,
         True if using intraday bars and no `timezone` specified; ignored for
         non-intraday bars, or if `timezone` is passed
 
+    master_fields : list of str, optional
+        [DEPRECATED] append these fields from the securities master database (pass ['?'] or any
+        invalid fieldname to see available fields). This parameter is deprecated and
+        will be removed in a future release. For better performance, use
+        `quantrocket.master.get_securities_reindexed_like` to get securities master
+        data shaped like prices.
+
     Returns
     -------
     DataFrame
@@ -655,6 +684,7 @@ def get_historical_prices(codes, start_date=None, end_date=None,
 
     db_universes = set()
     db_bar_sizes = set()
+    db_domains = set()
     for db in dbs:
         db_config = get_db_config(db)
         _db_universes = db_config.get("universes", None)
@@ -662,6 +692,8 @@ def get_historical_prices(codes, start_date=None, end_date=None,
             db_universes.update(set(_db_universes))
         bar_size = db_config.get("bar_size")
         db_bar_sizes.add(bar_size)
+        db_domain = db_config.get("domain", "main")
+        db_domains.add(db_domain)
 
     db_universes = list(db_universes)
     db_bar_sizes = list(db_bar_sizes)
@@ -672,6 +704,12 @@ def get_historical_prices(codes, start_date=None, end_date=None,
             "bar sizes: {1}".format(", ".join(dbs), ", ".join(db_bar_sizes))
         )
 
+    if len(db_domains) > 1:
+        raise ParameterError(
+            "all databases must use the same securities master domain but {0} "
+            "use different domains: {1}".format(", ".join(dbs), ", ".join(db_domains))
+        )
+
     all_prices = []
 
     for db in dbs:
@@ -692,46 +730,72 @@ def get_historical_prices(codes, start_date=None, end_date=None,
         tmp_filepath = "{0}/history.{1}.{2}.{3}.csv".format(
             TMP_DIR, db, os.getpid(), time.time()
         )
-        download_history_file(db, tmp_filepath, **kwargs)
+        try:
+            download_history_file(db, tmp_filepath, **kwargs)
+        except NoHistoricalData as e:
+            # don't complain about NoHistoricalData if we're checking
+            # multiple databases, unless none of them have data
+            if len(dbs) == 1:
+                raise
+            else:
+                continue
 
         prices = pd.read_csv(tmp_filepath)
         all_prices.append(prices)
 
         os.remove(tmp_filepath)
 
+    # complain if multiple dbs and none had data
+    if len(dbs) > 1 and not all_prices:
+        raise NoHistoricalData("no history matches the query parameters in any of {0}".format(
+            ", ".join(dbs)
+        ))
+
     prices = pd.concat(all_prices)
 
     prices = prices.pivot(index="ConId", columns="Date").T
     prices.index.set_names(["Field", "Date"], inplace=True)
 
+    master_fields = master_fields or []
+    if master_fields:
+        import warnings
+        # DeprecationWarning is ignored by default but we want the user
+        # to see it
+        warnings.simplefilter("always", DeprecationWarning)
+        warnings.warn(
+            "`master_fields` parameter is deprecated and will be removed in a "
+            "future release. For better performance, please use "
+            "`quantrocket.master.get_securities_reindexed_like` "
+            "to get securities master data shaped like prices.", DeprecationWarning)
+
+        if isinstance(master_fields, tuple):
+            master_fields = list(master_fields)
+        elif not isinstance(master_fields, list):
+            master_fields = [master_fields]
+
+    # master fields that are required internally but shouldn't be returned to
+    # the user (potentially Timezone)
+    internal_master_fields = []
 
     is_intraday = db_bar_sizes[0] not in ("1 day", "1 week", "1 month")
 
     if is_intraday and not timezone and infer_timezone is not False:
         infer_timezone = True
-        if not master_fields:
-            master_fields = []
-        if "Timezone" not in master_fields:
-            master_fields.append("Timezone")
+        if not master_fields or "Timezone" not in master_fields:
+            internal_master_fields.append("Timezone")
 
     # Next, get the master file
-    if master_fields:
-        universes = universes
-        conids = conids
-        if not conids and not universes:
-            universes = db_universes
-            if not universes:
-                conids = list(prices.columns)
+    if master_fields or internal_master_fields:
+        conids = list(prices.columns)
 
-        f = io.StringIO()
+        domain = list(db_domains)[0] if db_domains else None
+
+        f = six.StringIO()
         download_master_file(
             f,
             conids=conids,
-            universes=universes,
-            exclude_conids=exclude_conids,
-            exclude_universes=exclude_universes,
-            fields=master_fields,
-            delisted=True
+            fields=master_fields + internal_master_fields,
+            domain=domain
         )
         securities = pd.read_csv(f, index_col="ConId")
 
@@ -741,22 +805,36 @@ def get_historical_prices(codes, start_date=None, end_date=None,
         if "Etf" in securities.columns:
             securities.loc[:, "Etf"] = securities.Etf.astype(bool)
 
-        # Append securities, indexed to the min date, to allow easy ffill on demand
-        securities = pd.DataFrame(securities.T, columns=prices.columns)
-        securities.index.name = "Field"
-        idx = pd.MultiIndex.from_product(
-            (securities.index, [prices.index.get_level_values("Date").min()]),
-            names=["Field", "Date"])
+        # Infer timezone if needed
+        if not timezone and infer_timezone:
+            timezones = securities.Timezone.unique()
 
-        securities = securities.reindex(index=idx, level="Field")
-        prices = pd.concat((prices, securities))
+            if len(timezones) > 1:
+                raise ParameterError(
+                    "cannot infer timezone because multiple timezones are present "
+                    "in data, please specify timezone explicitly (timezones: {0})".format(
+                        ", ".join(timezones)))
+
+            timezone = timezones[0]
+
+        # Drop any internal-only fields
+        if internal_master_fields:
+            securities = securities.drop(internal_master_fields, axis=1)
+
+        if not securities.empty:
+            # Append securities, indexed to the min date, to allow easy ffill on demand
+            securities = pd.DataFrame(securities.T, columns=prices.columns)
+            securities.index.name = "Field"
+            idx = pd.MultiIndex.from_product(
+                (securities.index, [prices.index.get_level_values("Date").min()]),
+                names=["Field", "Date"])
+
+            securities = securities.reindex(index=idx, level="Field")
+            prices = pd.concat((prices, securities))
 
     if is_intraday:
         dates = pd.to_datetime(prices.index.get_level_values("Date"), utc=True)
 
-        if not timezone and infer_timezone:
-            timezone = _infer_timezone(prices)
-
         if timezone:
             dates = dates.tz_convert(timezone)
     else:
@@ -802,11 +880,11 @@ def get_historical_prices(codes, start_date=None, end_date=None,
     unique_times = prices.index.get_level_values("Time").unique()
     interpolated_index = None
     for field in unique_fields:
-        if field in master_fields:
+        if master_fields and field in master_fields:
             min_date = prices.loc[field].index.min()
             field_idx = pd.MultiIndex.from_tuples([(field,min_date[0], min_date[1])])
         else:
-            field_idx = pd.MultiIndex.from_product([[field], unique_dates, unique_times])
+            field_idx = pd.MultiIndex.from_product([[field], unique_dates, unique_times]).sort_values()
         if interpolated_index is None:
             interpolated_index = field_idx
         else:
@@ -856,4 +934,28 @@ def load_history_from_file(code, infilepath_or_buffer):
     return response.json()
 
 def _cli_load_history_from_file(*args, **kwargs):
-    return json_to_cli(load_history_from_file, *args, **kwargs)
\ No newline at end of file
+    return json_to_cli(load_history_from_file, *args, **kwargs)
+
+@deprecated_replaced_by(collect_history)
+def fetch_history(*args, **kwargs):
+    """
+    Collect historical market data from IB and save it to a history database.
+
+    [DEPRECATED] `fetch_history` is deprecated and will be removed
+    in a future release, please use `collect_history` instead.
+    """
+    return collect_history(*args, **kwargs)
+
+@deprecated_replaced_by("collect", old_name="fetch")
+def _cli_fetch_history(*args, **kwargs):
+    return json_to_cli(collect_history, *args, **kwargs)
+
+@deprecated_replaced_by(cancel_collections)
+def cancel_history_requests(*args, **kwargs):
+    """
+    Cancel running or pending historical data collections.
+
+    [DEPRECATED] `cancel_history_requests` is deprecated and will be removed
+    in a future release, please use `cancel_collections` instead.
+    """
+    return cancel_collections(*args, **kwargs)
