@@ -18,6 +18,7 @@ from quantrocket.houston import houston
 from quantrocket.cli.utils.output import json_to_cli
 from quantrocket.cli.utils.stream import to_bytes
 from quantrocket.cli.utils.files import write_response_to_filepath_or_buffer
+from quantrocket.utils.warn import deprecated_replaced_by
 
 def list_exchanges(regions=None, sec_types=None):
     """
@@ -41,27 +42,29 @@ def list_exchanges(regions=None, sec_types=None):
     if regions:
         params["regions"] = regions
 
-    response = houston.get("/master/exchanges", params=params)
+    response = houston.get("/master/exchanges", params=params, timeout=60)
     houston.raise_for_status_with_json(response)
     return response.json()
 
 def _cli_list_exchanges(*args, **kwargs):
     return json_to_cli(list_exchanges, *args, **kwargs)
 
-def fetch_listings(exchange=None, sec_types=None, currencies=None, symbols=None,
-                        universes=None, conids=None):
+def collect_listings(exchanges=None, sec_types=None, currencies=None, symbols=None,
+                     universes=None, conids=None, exchange=None):
     """
-    Fetch securities listings from IB into securities master database, either by exchange or by universes/conids.
+    Collect securities listings from IB and store in securities master
+    database (quantrocket.master.main.sqlite).
 
-
-    Specify an exchange (optionally filtering by security type, currency, and/or symbol) to fetch
-    listings from the IB website and fetch associated contract details from the IB API. Or, specify universes
-    or conids to fetch details from the IB API, bypassing the website.
+    Specify an exchange (optionally filtering by security type, currency,
+    and/or symbol) to collect listings from the IB website and collect
+    associated contract details from the IB API. Or, specify universes or
+    conids to collect details from the IB API, bypassing the website.
 
     Parameters
     ----------
-    exchange : str
-        the exchange code to fetch listings for (required unless providing universes or conids)
+    exchanges : list or str
+        one or more exchange codes to collect listings for (required unless providing
+        universes or conids)
 
     sec_types : list of str, optional
         limit to these security types. Possible choices: STK, ETF, FUT, CASH, IND
@@ -78,6 +81,11 @@ def fetch_listings(exchange=None, sec_types=None, currencies=None, symbols=None,
     conids : list of int, optional
         limit to these conids
 
+    exchange : str
+        DEPRECATED, this option will be removed in a future release, please use
+        `exchanges` instead (previously only a single exchange was supported but
+        now multiple exchanges are supported)
+
     Returns
     -------
     dict
@@ -85,8 +93,8 @@ def fetch_listings(exchange=None, sec_types=None, currencies=None, symbols=None,
 
     """
     params = {}
-    if exchange:
-        params["exchange"] = exchange
+    if exchanges:
+        params["exchanges"] = exchanges
     if sec_types:
         params["sec_types"] = sec_types
     if currencies:
@@ -97,17 +105,54 @@ def fetch_listings(exchange=None, sec_types=None, currencies=None, symbols=None,
         params["universes"] = universes
     if conids:
         params["conids"] = conids
+    if exchange:
+        import warnings
+        # DeprecationWarning is ignored by default but we want the user
+        # to see it
+        warnings.simplefilter("always", DeprecationWarning)
+        warnings.warn(
+            "the `exchange` option is deprecated and will be removed in a "
+            "future release, please use `exchanges` instead (previously only "
+            "a single exchange was supported but now multiple exchanges are "
+            "supported)", DeprecationWarning)
+        params["exchange"] = exchange
 
-    response = houston.post("/master/listings", params=params)
+    response = houston.post("/master/securities", params=params)
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def _cli_fetch_listings(*args, **kwargs):
-    return json_to_cli(fetch_listings, *args, **kwargs)
+def _cli_collect_listings(*args, **kwargs):
+    return json_to_cli(collect_listings, *args, **kwargs)
 
-def fetch_option_chains(universes=None, conids=None, infilepath_or_buffer=None):
+def collect_sharadar_listings():
     """
-    Fetch option chains for underlying securities.
+    Collect securities listings from Sharadar and save to
+    quantrocket.master.sharadar.sqlite.
+
+    Requires a Sharadar data plan. Collects NYSE, NASDAQ, or all US stock
+    listings, depending on your plan.
+
+    Sharadar listings have their own ConIds which are distinct from IB ConIds.
+    To facilitate using Sharadar and IB data together or separately, this command
+    also collects a list of IB<->Sharadar ConId translations and saves them
+    to quantrocket.master.translations.sqlite. They can be queried via
+    `translate_conids`.
+
+    Returns
+    -------
+    dict
+        status message
+    """
+    response = houston.post("/master/sharadar/securities")
+    houston.raise_for_status_with_json(response)
+    return response.json()
+
+def _cli_collect_sharadar_listings(*args, **kwargs):
+    return json_to_cli(collect_sharadar_listings, *args, **kwargs)
+
+def collect_option_chains(universes=None, conids=None, infilepath_or_buffer=None):
+    """
+    Collect option chains for underlying securities.
 
     Note: option chains often consist of hundreds, sometimes thousands of
     options per underlying security. Be aware that requesting option chains
@@ -120,13 +165,14 @@ def fetch_option_chains(universes=None, conids=None, infilepath_or_buffer=None):
     Parameters
     ----------
     universes : list of str, optional
-        fetch options for these universes of underlying securities
+        collect options for these universes of underlying securities
 
     conids : list of int, optional
-        fetch options for these underlying conids
+        collect options for these underlying conids
 
     infilepath_or_buffer : str or file-like object, optional
-        fetch options for the conids in this file (specify '-' to read file from stdin)
+        collect options for the conids in this file (specify '-' to read file
+        from stdin)
 
     Returns
     -------
@@ -156,14 +202,15 @@ def fetch_option_chains(universes=None, conids=None, infilepath_or_buffer=None):
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def _cli_fetch_option_chains(*args, **kwargs):
-    return json_to_cli(fetch_option_chains, *args, **kwargs)
+def _cli_collect_option_chains(*args, **kwargs):
+    return json_to_cli(collect_option_chains, *args, **kwargs)
 
 def diff_securities(universes=None, conids=None, infilepath_or_buffer=None,
-                    fields=None, delist_missing=False, delist_exchanges=None, wait=False):
+                    fields=None, delist_missing=False, delist_exchanges=None,
+                    wait=False):
     """
     Flag security details that have changed in IB's system since the time they
-    were last loaded into the securities master database.
+    were last collected into the securities master database.
 
     Diff can be run synchronously or asynchronously (asynchronous is the default
     and is recommended if diffing more than a handful of securities).
@@ -238,7 +285,8 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
                          currencies=None, universes=None, symbols=None, conids=None,
                          exclude_universes=None, exclude_conids=None,
                          sectors=None, industries=None, categories=None,
-                         delisted=False, frontmonth=False, fields=None):
+                         exclude_delisted=False, delisted=True, frontmonth=False, fields=None,
+                         domain=None):
     """
     Query security details from the securities master database and download to file.
 
@@ -283,8 +331,13 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
     categories : list of str, optional
         limit to these categories
 
+    exclude_delisted : bool
+        exclude delisted securities (default is to include them)
+
     delisted : bool
-        include delisted securities (default False)
+        [DEPRECATED] include delisted securities; this parameter is deprecated
+        and will be removed in a future release; it has no effect as delisted
+        securities are included by default
 
     frontmonth : bool
         exclude backmonth and expired futures contracts (default False)
@@ -293,18 +346,35 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
         only return these fields (pass ['?'] or any invalid fieldname to see
         available fields)
 
+    domain : str, optional
+        query against this domain (default is 'main', which runs against
+        quantrocket.master.main.sqlite. Possible choices: main, sharadar)
+
     Returns
     -------
     None
 
     Examples
     --------
-    You can use StringIO to load the CSV into pandas.
+    Download several exchanges to file:
+
+    >>> download_master_file("securities.csv", exchanges=["NYSE","NASDAQ"])
+
+    Download securities for a particular universe to in-memory file and
+    load the CSV into pandas.
 
     >>> f = io.StringIO()
     >>> download_master_file(f, universes=["my-universe"])
     >>> securities = pd.read_csv(f)
+
+    Download Sharadar securities from quantrocket.master.sharadar.sqlite:
+
+    >>> download_master_file("sharadar_securities.csv", domain="sharadar")
     """
+    # Handle legacy param "delisted"
+    if delisted is False and exclude_delisted is None:
+        exclude_delisted = True
+
     params = {}
     if exchanges:
         params["exchanges"] = exchanges
@@ -328,8 +398,8 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
         params["industries"] = industries
     if categories:
         params["categories"] = categories
-    if delisted:
-        params["delisted"] = delisted
+    if exclude_delisted:
+        params["exclude_delisted"] = exclude_delisted
     if frontmonth:
         params["frontmonth"] = frontmonth
     if fields:
@@ -337,10 +407,14 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
 
     output = output or "csv"
 
+    url = "/master/{0}securities.{1}".format(
+        "{0}/".format(domain) if domain else "",
+        output)
+
     if output not in ("csv", "json", "txt"):
         raise ValueError("Invalid ouput: {0}".format(output))
 
-    response = houston.get("/master/securities.{0}".format(output), params=params)
+    response = houston.get(url, params=params)
 
     houston.raise_for_status_with_json(response)
 
@@ -351,8 +425,136 @@ def download_master_file(filepath_or_buffer=None, output="csv", exchanges=None,
 def _cli_download_master_file(*args, **kwargs):
     return json_to_cli(download_master_file, *args, **kwargs)
 
+def get_securities_reindexed_like(reindex_like, domain, fields=None):
+    """
+    Return a multiindex DataFrame of securities master data, reindexed to
+    match the index and columns (conids) of `reindex_like`.
+
+    Parameters
+    ----------
+    reindex_like : DataFrame, required
+        a DataFrame (usually of prices) with dates for the index and conids
+        for the columns, to which the shape of the resulting DataFrame will
+        be conformed
+
+    domain : str, required
+        the domain of the conids in `reindex_like`, and the securities master domain
+        to be queried. Possible choices: main, sharadar
+
+    fields : list of str
+        a list of fields to include in the resulting DataFrame. Defaults to
+        including all fields. For faster performance, limiting fields to
+        those needed is highly recommended, especially for large universes.
+
+    Returns
+    -------
+    DataFrame
+        a multiindex (Field, Date) DataFrame of securities master data, shaped
+        like the input DataFrame
+
+    Examples
+    --------
+    Get symbols, currencies, and primary exchanges using a DataFrame of
+    historical prices from IB:
+
+    >>> closes = prices.loc["Close"]
+    >>> securities = get_securities_reindexed_like(
+            closes, domain="main",
+            fields=["PrimaryExchange"])
+    >>> exchanges = securities.loc["PrimaryExchange"]
+    >>> nyse_closes = closes.where(exchanges == "NYSE")
+    """
+    try:
+        import pandas as pd
+    except ImportError:
+        raise ImportError("pandas must be installed to use this function")
+
+    conids = list(reindex_like.columns)
+
+    f = six.StringIO()
+    download_master_file(f, domain=domain, conids=conids, fields=fields)
+    securities = pd.read_csv(f, index_col="ConId")
+
+    all_master_fields = {}
+
+    for col in securities.columns:
+        this_col = securities[col]
+        if col in ("Delisted", "Etf"):
+            this_col = this_col.astype(bool)
+        all_master_fields[col] = reindex_like.apply(lambda x: this_col, axis=1)
+
+    names = list(reindex_like.index.names)
+    names.insert(0, "Field")
+
+    securities = pd.concat(all_master_fields, names=names)
+    return securities
+
+def translate_conids(conids, from_domain=None, to_domain=None):
+    """
+    Translate conids (contract IDs) from one domain to another.
+
+    Only translations to and from the "main" domain (that is, the
+    IB domain) are supported.
+
+    Parameters
+    ----------
+    conids : list of int, required
+        the conids to translate
+
+    from_domain : str, optional
+        the domain to translate from. This is the domain of the provided
+        conids. Possible choices: main, sharadar
+
+    to_domain : str, optional
+        the domain to translate to. Possible choices: main, sharadar
+
+    Returns
+    -------
+    dict
+        dict of <from_domain conid>:<to_domain conid>
+
+    Examples
+    --------
+    Translate a DataFrame with IB conids as columns to one with Sharadar
+    conids as columns, and mask columns that can't be translated:
+
+    >>> ib_conids = list(df_with_ib_cols.columns)
+    >>> ib_to_sharadar = translate_conids(ib_conids, to_domain="sharadar")
+    >>> df_with_sharadar_cols = df_with_ib_cols.rename(columns=ib_to_sharadar)
+    >>> # Mask columns where no Sharadar conid was available
+    >>> no_translations = set(ib_conids) - set(ib_to_sharadar)
+    >>> df_with_sharadar_cols.loc[:, no_translations] = None
+
+    Translate a DataFrame with Sharadar conids as columns to one with IB
+    conids as columns, and drop columns that can't be translated:
+
+    >>> sharadar_conids = list(df_with_sharadar_cols.columns)
+    >>> sharadar_to_ib = translate_conids(sharadar_conids, from_domain="sharadar")
+    >>> df_with_ib_cols = df_with_sharadar_cols.rename(columns=sharadar_to_ib)
+    >>> # Drop columns where no IB conid was available
+    >>> no_translations = set(sharadar_conids) - set(sharadar_to_ib)
+    >>> df_with_ib_cols = df_with_ib_cols.drop(no_translations, axis=1)
+    """
+    params = {}
+    params["conids"] = conids
+    if from_domain:
+        params["from_domain"] = from_domain
+    if to_domain:
+        params["to_domain"] = to_domain
+
+    response = houston.get("/master/translations", params=params)
+    houston.raise_for_status_with_json(response)
+    translations = response.json()
+    # JSON requires dict keys to be strings, re-cast to int
+    translations = dict([(int(k),v) for k,v in translations.items()])
+    return translations
+
+def _cli_translate_conids(*args, **kwargs):
+    return json_to_cli(translate_conids, *args, **kwargs)
+
 def create_universe(code, infilepath_or_buffer=None, from_universes=None,
-                    exclude_delisted=False, append=False, replace=False):
+                    exclude_delisted=False, append=False, replace=False,
+                    domain=None):
     """
     Create a universe of securities.
 
@@ -368,7 +570,8 @@ def create_universe(code, infilepath_or_buffer=None, from_universes=None,
         create the universe from these existing universes
 
     exclude_delisted : bool
-        exclude delisted securities that would otherwise be included (default is not to exclude them)
+        exclude delisted securities that would otherwise be included (default
+        is not to exclude them)
 
     append : bool
         append to universe if universe already exists (default False)
@@ -376,11 +579,27 @@ def create_universe(code, infilepath_or_buffer=None, from_universes=None,
     replace : bool
         replace universe if universe already exists (default False)
 
+    domain : str, optional
+        create universe in this domain (default is 'main', which runs against
+        quantrocket.master.main.sqlite. Possible choices: main, sharadar)
+
     Returns
     -------
     dict
         status message
 
+    Examples
+    --------
+    Create a universe called 'nyse-stk' from a CSV file:
+
+    >>> create_universe("usa-stk", "nyse_securities.csv")
+
+    Create a universe from a DataFrame:
+
+    >>> import io
+    >>> f = io.StringIO()
+    >>> japan_securities.to_csv(f)
+    >>> create_universe("japan-stk", f)
     """
     if append and replace:
         raise ValueError("append and replace are mutually exclusive")
@@ -393,7 +612,9 @@ def create_universe(code, infilepath_or_buffer=None, from_universes=None,
     if replace:
         params["replace"] = replace
 
-    url = "/master/universes/{0}".format(code)
+    url = "/master/{0}universes/{1}".format(
+        "{0}/".format(domain) if domain else "",
+        code)
 
     if append:
         method = "PATCH"
@@ -420,38 +641,60 @@ def create_universe(code, infilepath_or_buffer=None, from_universes=None,
 def _cli_create_universe(*args, **kwargs):
     return json_to_cli(create_universe, *args, **kwargs)
 
-def delete_universe(code):
+def delete_universe(code, domain=None):
     """
-    Delete a universe. (The listings details of the member securities won't be deleted,
-    only their grouping as a universe).
+    Delete a universe.
+
+    The listings details of the member securities won't be deleted, only
+    their grouping as a universe.
 
     Parameters
     ----------
     code : str, required
         the universe code
 
+    domain : str, optional
+        the domain from which to delete the universe (default is 'main', which
+        runs against quantrocket.master.main.sqlite. Possible choices:
+        main, sharadar)
+
     Returns
     -------
     dict
         status message
     """
-    response = houston.delete("/master/universes/{0}".format(code))
+
+    url = "/master/{0}universes/{1}".format(
+        "{0}/".format(domain) if domain else "",
+        code)
+
+    response = houston.delete(url)
     houston.raise_for_status_with_json(response)
     return response.json()
 
 def _cli_delete_universe(*args, **kwargs):
     return json_to_cli(delete_universe, *args, **kwargs)
 
-def list_universes():
+def list_universes(domain=None):
     """
     List universes and their size.
 
+    Parameters
+    ----------
+    domain : str, optional
+        the domain to list universes for (default is 'main', which
+        runs against quantrocket.master.main.sqlite. Possible choices:
+        main, sharadar)
+
     Returns
     -------
     dict
         dict of universe:size
     """
-    response = houston.get("/master/universes")
+    url = "/master/{0}universes".format(
+        "{0}/".format(domain) if domain else "")
+
+    response = houston.get(url)
     houston.raise_for_status_with_json(response)
     return response.json()
 
@@ -462,9 +705,9 @@ def delist_security(conid=None, symbol=None, exchange=None, currency=None, sec_t
     """
     Mark a security as delisted.
 
-    The security can be specified by conid or a combination of other parameters (for
-    example, symbol + exchange). As a precaution, the request will fail if the parameters
-    match more than one security.
+    The security can be specified by conid or a combination of other
+    parameters (for example, symbol + exchange). As a precaution, the request
+    will fail if the parameters match more than one security.
 
     Parameters
     ----------
@@ -481,8 +724,8 @@ def delist_security(conid=None, symbol=None, exchange=None, currency=None, sec_t
         the currency of the security to be delisted (if needed to disambiguate)
 
     sec_type : str, optional
-        the security type of the security to be delisted (if needed to disambiguate). Possible
-        choices: STK, ETF, FUT, CASH, IND
+        the security type of the security to be delisted (if needed to disambiguate).
+        Possible choices: STK, ETF, FUT, CASH, IND
 
     Returns
     -------
@@ -549,9 +792,10 @@ def _cli_load_or_show_rollrules(filename=None):
     else:
         return json_to_cli(get_rollrules_config)
 
-def fetch_calendar(exchanges=None):
+def collect_calendar(exchanges=None):
     """
-    Fetch upcoming trading hours for exchanges and save to securites master database.
+    Collect upcoming trading hours for exchanges and save to securites
+    master database.
 
     Parameters
     ----------
@@ -571,8 +815,8 @@ def fetch_calendar(exchanges=None):
     houston.raise_for_status_with_json(response)
     return response.json()
 
-def _cli_fetch_calendar(*args, **kwargs):
-    return json_to_cli(fetch_calendar, *args, **kwargs)
+def _cli_collect_calendar(*args, **kwargs):
+    return json_to_cli(collect_calendar, *args, **kwargs)
 
 def list_calendar_statuses(exchanges, sec_type=None, in_=None, ago=None, outside_rth=False):
     """
@@ -786,4 +1030,43 @@ def round_to_tick_sizes(infilepath_or_buffer, round_fields,
     write_response_to_filepath_or_buffer(filepath_or_buffer, response)
 
 def _cli_round_to_tick_sizes(*args, **kwargs):
-    return json_to_cli(round_to_tick_sizes, *args, **kwargs)
\ No newline at end of file
+    return json_to_cli(round_to_tick_sizes, *args, **kwargs)
+
+@deprecated_replaced_by(collect_listings)
+def fetch_listings(*args, **kwargs):
+    """
+    Collect securities listings from IB into securities master database, either by exchange or by universes/conids.
+
+    [DEPRECATED] `fetch_listings` is deprecated and will be removed
+    in a future release, please use `collect_listings` instead.
+    """
+    return collect_listings(*args, **kwargs)
+
+@deprecated_replaced_by(collect_option_chains)
+def fetch_option_chains(*args, **kwargs):
+    """
+    Collect option chains for underlying securities.
+
+    [DEPRECATED] `fetch_option_chains` is deprecated and will be removed
+    in a future release, please use `collect_option_chains` instead.
+    """
+    return collect_option_chains(*args, **kwargs)
+
+@deprecated_replaced_by(collect_calendar)
+def fetch_calendar(*args, **kwargs):
+    """
+    Collect upcoming trading hours for exchanges and save to securites
+    master database.
+
+    [DEPRECATED] `fetch_calendar` is deprecated and will be removed
+    in a future release, please use `collect_calendar` instead.
+    """
+    return collect_calendar(*args, **kwargs)
+
+@deprecated_replaced_by("collect-calendar", old_name="fetch-calendar")
+def _cli_fetch_calendar(*args, **kwargs):
+    return json_to_cli(collect_calendar, *args, **kwargs)
+
+@deprecated_replaced_by("collect", old_name="listings")
+def _cli_listings(*args, **kwargs):
+    return json_to_cli(collect_listings, *args, **kwargs)
